from rest_framework import serializers
from authentication.models import CustomUser as User
from django.contrib import auth
from rest_framework.exceptions import AuthenticationFailed
from rest_framework_simplejwt.tokens import RefreshToken, TokenError
import datetime


class RegisterSerializer(serializers.ModelSerializer):
    password = serializers.CharField(max_length=68, min_length=8, write_only=True)
    name = serializers.CharField(required=True)

    class Meta:
        model = User
        fields = ["name", "email", "password"]

    def validate(self, attrs):
        email = attrs.get("email", "")
        name = attrs.get("name", "")
        if not name:
            raise serializers.ValidationError({"name": "This field is required."})
        return attrs

    def create(self, validated_data):
        return User.objects.create_user(**validated_data)


class LoginSerializer(serializers.ModelSerializer):
    password = serializers.CharField(max_length=68, min_length=8, write_only=True)
    email = serializers.EmailField(
        max_length=255,
    )
    tokens = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(read_only=True)
    updated_at = serializers.DateTimeField(read_only=True)
    name = serializers.CharField(max_length=68, min_length=8, read_only=True)

    def get_tokens(self, obj):
        user = User.objects.get(email=obj["email"])
        return {"refresh": user.tokens()["refresh"], "access": user.tokens()["access"]}

    class Meta:
        model = User
        fields = [
            "name",
            "password",
            "email",
            "tokens",
            "is_subscription_active",
            "is_trial",
            "created_at",
            "updated_at"
        ]

    def validate(self, attrs):
        email = attrs.get("email", "")
        password = attrs.get("password", "")

        user = auth.authenticate(email=email, password=password)
        if not user:
            raise AuthenticationFailed("Invalid credentials, try again")
        if not user.is_active:
            raise AuthenticationFailed("Account disabled, contact admin")
        if (
            user
            and user.is_trial
            and (datetime.datetime.now().date() - user.created_at.date()).days >= 14
        ):
            user.is_trial = False
            user.save()
        return {
            "name": user.name,
            "email": user.email,
            "tokens": user.tokens,
            "is_subscription_active": user.is_subscription_active,
            "is_trial": user.is_trial,
            "created_at": str(user.created_at.date()),
            "updated_at": str(user.updated_at.date()),
        }


class LogoutSerializer(serializers.Serializer):
    refresh = serializers.CharField()

    def validate(self, attrs):
        self.token = attrs["refresh"]
        return attrs

    def save(self, **kwargs):
        try:
            RefreshToken(self.token).blacklist()
        except TokenError:
            self.fail("bad_token")


class ResetPasswordSerializer(serializers.ModelSerializer):
    new_password = serializers.CharField()

    class Meta:
        model = User
        fields = ["email", "new_password"]

    def validate_email(self, value):
        if not User.objects.filter(email=value).exists():
            raise serializers.ValidationError("No user found with this email address.")
        return value

    def save(self, **kwargs):
        email = self.email
        new_password = self.new_password
        user = User.objects.get(email=email)
        user.password = set_password(new_password)
        user.save()



from rest_framework_simplejwt.exceptions import InvalidToken, TokenError
from django.core.exceptions import ObjectDoesNotExist
from rest_framework_simplejwt.tokens import AccessToken
from .models import CustomUser
import requests
from django.conf import settings

def get_user_from_token(token):
    try:
        # Decode the token
        decoded_token = AccessToken(token)
        # Get the user ID
        user_id = decoded_token['user_id']
        # Get the user instance
        user = CustomUser.objects.get(id=user_id)
        return user
    except (InvalidToken, TokenError):
        # Handle invalid or expired token
        print("Invalid or expired token.")
    except ObjectDoesNotExist:
        # Handle non-existent user
        print("User does not exist.")



import os
import platform
import subprocess
import time
import shutil


def create_django_app(app_name, database):
    # Use os.path.join to join paths in a platform-independent way
    app_path = os.path.join(os.getcwd(), app_name)
    venv_path = os.path.join(app_path, "venv")

    # Create the app directory if it doesn't exist
    os.makedirs(app_path, exist_ok=True)

    # Move to the app directory
    os.chdir(app_path)

    # Use subprocess.run for running commands to capture the output and handle errors
    subprocess.run(["python", "-m", "venv", "venv"], shell=True, check=True)

    # Use os.path.join to create the path to the activate script
    activate_script = os.path.join(
        venv_path, "Scripts" if platform.system() == "Windows" else "bin", "activate"
    )

    # Use the "source" command on Linux/Mac or "call" on Windows to activate the virtual environment
    activate_command = (
        f"call {activate_script}"
        if platform.system() == "Windows"
        else f"source {activate_script}"
    )
    subprocess.run(activate_command, shell=True, check=True)

    manage_script = os.path.join(app_path, "manage.py")

    subprocess.run(["pip", "install", "django"], shell=True, check=True)
    subprocess.run(
        ["python", "manage.py", "startapp", "authentication"], shell=True, check=True
    )

    # Install Django Rest Framework (DRF)
    subprocess.run(
        [
            "pip",
            "install",
            "djangorestframework",
            "djangorestframework-simplejwt",
            "python-dotenv",
        ],
        shell=True,
        check=True,
    )

    # Update settings.py with DRF in INSTALLED_APPS
    with open(os.path.join(app_path, app_name, "settings.py"), "a") as settings_file:
        settings_file.write(
            "\nINSTALLED_APPS += ['rest_framework', 'rest_framework_simplejwt', 'authentication']"
        )

    with open(os.path.join(app_path, app_name, "settings.py"), "a") as settings_file:
        settings_file.write("\nAUTH_USER_MODEL = 'authentication.CustomUser'")

    # Update urls.py with DRF
    with open(os.path.join(app_path, app_name, "urls.py"), "a") as urls_file:
        urls_file.write(
            "\nfrom django.urls import include\nfrom rest_framework_simplejwt.views import (TokenObtainPairView,TokenRefreshView,)\n\nurlpatterns += [path('api-auth/', include('rest_framework.urls')),\
                        path('api/token/', TokenObtainPairView.as_view(), name='token_obtain_pair'),                            \
                        path('api/token/refresh/', TokenRefreshView.as_view(), name='token_refresh'), \
                        path('api/', include('authentication.urls')),]"
        )

    # Update requirements.txt
    subprocess.run(["pip", "freeze", ">", "requirements.txt"], shell=True, check=True)

    parent_directory = os.path.abspath(os.path.dirname(__file__))
    api_folder_path = os.path.join(app_path, "authentication", "api")
    os.makedirs(api_folder_path)

    # models.py
    with open(
        os.path.join(parent_directory, "authentication", "models.py"), "r"
    ) as models_file:
        models_file_content = models_file.read()
        with open(
            os.path.join(app_path, "authentication", "models.py"), "a+"
        ) as new_models_file:
            new_models_file.write(models_file_content)

    # serializers.py
    with open(
        os.path.join(parent_directory, "authentication","serializers.py"), "r"
    ) as serializers_file:
        serializers_file_content = serializers_file.read()
        with open(
            os.path.join(api_folder_path, "serializers.py"), "a+"
        ) as new_serializers_file:
            new_serializers_file.write(serializers_file_content)

    # views.py
    with open(
        os.path.join(parent_directory, "authentication",  "views.py"), "r"
    ) as views_file:
        views_file_content = views_file.read()
        with open(os.path.join(api_folder_path, "views.py"), "a+") as new_views_file:
            new_views_file.write(views_file_content)

    # urls.py
    with open(
        os.path.join(parent_directory, "authentication", "urls.py"), "r"
    ) as urls_file:
        urls_file_content = urls_file.read()
        with open(
            os.path.join(app_path, "authentication", "urls.py"), "a+"
        ) as new_urls_file:
            new_urls_file.write(urls_file_content)

    if database == "Sqlite3":
        subprocess.run(["python", "manage.py", "makemigrations", "--noinput"], check=True)
        subprocess.run(["python", "manage.py", "migrate"], shell=True, check=True)



from django.db import models
from django.core.validators import RegexValidator
from django.core.exceptions import ValidationError
from django.contrib.auth.base_user import BaseUserManager
from django.contrib.auth.models import AbstractUser
from rest_framework_simplejwt.tokens import RefreshToken
from django.utils.timezone import now
import datetime


GENDER_CHOICES  = (('M', 'Male'), ('F', 'Female'), ('O', 'Other'))

contact_validator = [
    RegexValidator(regex=r'^[6-9]{1}[0-9]{9}$', message="Please enter a valid 10-digit mobile number.")
]

def regex_validators(value):
    err = None
    for validator in contact_validator:
        try:
            validator(value)
            return value
        except ValidationError as exc:
            err = exc
    raise err

class CustomUserManager(BaseUserManager):
    def create_user(self, email, password, **extra_fields):
        if not email:
            raise ValueError('The Email must be set')
        
        email = self.normalize_email(email)
        existing_user = self.model.objects.filter(email=email).exists()
        
        if existing_user:
            raise ValueError('A user with this email already exists')
        
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError(_('Superuser must have is_staff=True.'))
        if extra_fields.get('is_superuser') is not True:
            raise ValueError(_('Superuser must have is_superuser=True.'))
        return self.create_user(email, password, **extra_fields)

class CustomUser(AbstractUser):
    username = None
    """ Required Parameters """
    name    = models.CharField(('Full Name'),max_length=100, blank=True)
    email   = models.EmailField('Email Address', unique=True)
    mobile  = models.CharField(('Mobile'), max_length=10, validators=[regex_validators,])

    """ Optional Profile Details  """
    dob = models.DateField(('Date of birth'), blank=True, null=True)
    avatar  = models.FileField(('Profile Picture'), blank=True, null=True)
    gender  = models.CharField(('Gender'), blank=True, max_length=1, choices=GENDER_CHOICES)
    state   = models.CharField(('State'),  blank=True, max_length=30)
    country = models.CharField(('Country'), max_length=200, blank=True, default="India")


    """ Account level details """
    is_subscription_active = models.BooleanField("Is Subscription Active", default=False)
    is_trial = models.BooleanField("Is Trial Period", default=True)
    is_admin    = models.BooleanField("Is Admin", blank=True, null=True, default=False)
    is_verified = models.BooleanField("Is Verified",blank=True, null=True, default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    objects = CustomUserManager()

    def __str__(self):
        return self.email

    def tokens(self):
        refresh = RefreshToken.for_user(self)
        return{
            'refresh':str(refresh),
            'access':str(refresh.access_token)
        }


from django.apps import AppConfig


class AuthenticationConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "authentication"

    def ready(self):
        import authentication.signals



from . import views
from authentication.api import views as api_view
from django.urls import path, include
from rest_framework.urlpatterns import format_suffix_patterns
from rest_framework_simplejwt.views import (
    TokenRefreshView,
)

from rest_framework_simplejwt.views import (
    TokenObtainPairView,
)
from django.contrib.auth.decorators import login_required

app_name = "api"


urlpatterns = [
    path("token/", TokenObtainPairView.as_view(), name="token_obtain_pair"),
    path("register/", api_view.RegisterView.as_view(), name="register"),
    path("login/", api_view.LoginAPIView.as_view(), name="login"),
    path("logout/", api_view.LogoutAPIView.as_view(), name="logout"),
    path("api/token/refresh/", TokenRefreshView.as_view(), name="token_refresh"),
]



import argparse
import os
from shutil import copytree, rmtree
import inquirer
import re
from art import *
from pathlib import Path
from src import utils




def configure_django_app(app_name, use_drf, database):
    print(f"Configuring Django app with the following options:")
    print(f"App Name: {app_name}")
    print(f"Use DRF: {use_drf}")
    print(f"Database: {database}\n\n")

    # Create a new Django project
    os.system(f"django-admin startproject {app_name}")

    # Modify settings.py based on user choices
    settings_path = os.path.join(app_name, app_name, "settings.py")

    with open(settings_path, "r") as settings_file:
        content = settings_file.read()

    if use_drf == "Yes (Recommended)":
        utils.create_django_app(app_name, database)
        return True


    if database == "Sqlite3":
        return True

    elif database == "Postgres":
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.postgresql',
                'NAME': app_name,
                'USER': app_name,
                'PASSWORD': '123cls4',
                'HOST': 'localhost',
                'PORT': '5432',
            }
        }

    elif database == "MySQL":
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.mysql',
                'NAME': app_name,
                'USER': app_name,
                'PASSWORD': '1234',
                'HOST': 'localhost',
                'PORT': '3306',
            }
        }

    # Use regular expressions to find and replace the DATABASES block
    pattern = re.compile(r'DATABASES\s*=\s*{[^}]+}\s*}', re.DOTALL)
    content = pattern.sub(f"DATABASES = {DATABASES}", content)

    with open(settings_path, "w") as settings_file:
        settings_file.write(content)



def main():
    Art=text2art("CEDRIC")
    print(Art)
    print(
        "🚀 Welcome to CEDRIC Django Template – because even coding wizards need a magic wand! 🧙\n\n"
        "#############################################################################################################################################\n"
        "# Your Django development journey is about to get as smooth as a salsa dancer on roller skates!                                             \n"
        "# Developed by: Abhishek Choudhury (Got so annoyed, decided to create his own helper library, but cool enough to share it with you! 😎)    \n"
        "# Get ready to soar through your coding adventure with a sprinkle of Python magic!\n#\n"
        "# Remember, bugs are just undocumented features waiting to be discovered! 🐞✨\n"
        "#############################################################################################################################################\n\n"
    )

    questions = [
        inquirer.Text(
            "app_name", message="What shall we call your magical application?"
        ),
        #inquirer.List(
        #    "use_docker",
        #    message="Are you ready to embark on a 🚢 Docker adventure?",
        #    choices=["Yes", "No"],
        #    default="Yes",
        #),
        inquirer.List(
            "use_drf",
            message="Shall we add Django DRF with JWT based APIs for authentication?\n (Note: This action will create a new app authentication in the django app, with a custom user model along with required fields like email, name, password)",
            choices=["Yes (Recommended)", "No"],
            default="Yes (Recommended)",
        ),
        #inquirer.List(
        #    "use_jwt",
        #    message="Shall we add JWT based authentication?",
        #    choices=["Yes (Recommended)", "No"],
        #    default="Yes (Recommended)",
        #),
        inquirer.List(
            "database",
            message="Select a Database for your coding kingdom",
            choices=["Postgres", "MySQL", "Sqlite3"],
            default="Sqlite3",
        ),
    ]

    answers = inquirer.prompt(questions)

    # Validate app name
    app_name = answers["app_name"]
    if not app_name or not app_name.isidentifier() or app_name[0].isdigit():
        print(
            "Error: Please provide a valid app name. It should be a valid Python identifier."
        )
        return

    # Confirm overwrite if the app directory already exists
    app_directory = os.path.join(os.getcwd(), app_name)
    if os.path.exists(app_directory):
        response = inquirer.confirm(
            f"The directory {app_name} already exists. Do you want to overwrite it?",
            default=True,
        )
        if not response:
            print("Aborting setup.")
            return
        else:
            rmtree(app_directory)

    configure_django_app(
        app_name,
        answers["use_drf"],
        answers["database"],
    )

    print("\n\nTa-da! Your Django app has been summoned successfully. 🚀✨ Now go forth and conjure some code magic!")


if __name__ == "__main__":
    main()



from rest_framework import generics, status, views, permissions
from rest_framework.response import Response
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from .serializers import RegisterSerializer, LoginSerializer, LogoutSerializer, ResetPasswordSerializer
from django.core.mail import send_mail
from rest_framework.views import APIView
from authentication.models import CustomUser
from django.contrib.auth.hashers import make_password

class RegisterView(generics.GenericAPIView):
    serializer_class = RegisterSerializer

    def post(self, request):
        user = request.data
        serializer = self.serializer_class(data=user)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        user_data = serializer.data
        return Response(user_data, status=status.HTTP_201_CREATED)


class LoginAPIView(generics.GenericAPIView):
    serializer_class = LoginSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class LogoutAPIView(generics.GenericAPIView):
    serializer_class = LogoutSerializer
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(status=status.HTTP_204_NO_CONTENT)


@api_view(['POST'])
def reset_password(request):
    email = request.data.get("email")
    password = request.data.get("new_password")
    user_obj = CustomUser.objects.filter(email=email).exists()
    if not len(password) >= 8:
        return Response({"data":"Password Should be greater or equal to 8 characters"}, status=status.HTTP_400_BAD_REQUEST)
    if not user_obj:
        return Response({"Error":"Email Address could not be found."}, status=status.HTTP_400_BAD_REQUEST)
    user_obj = CustomUser.objects.get(email=email)
    user_obj.password = make_password(password)
    user_obj.save()
    return Response({"data":"Password Reset Successful"}, status=status.HTTP_201_CREATED)



from django import forms
from django.contrib import admin
from django.contrib.auth.models import Group
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.core.exceptions import ValidationError

from authentication.models import *


class UserCreationForm(forms.ModelForm):
    """A form for creating new users. Includes all the required
    fields, plus a repeated password."""

    password1 = forms.CharField(label="Password", widget=forms.PasswordInput)
    password2 = forms.CharField(
        label="Password confirmation", widget=forms.PasswordInput
    )

    class Meta:
        model = CustomUser
        fields = ["email", "password"]

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise ValidationError("Passwords don't match")
        return password2

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class UserChangeForm(forms.ModelForm):
    """A form for updating users. Includes all the fields on
    the user, but replaces the password field with admin's
    disabled password hash display field.
    """

    password = ReadOnlyPasswordHashField()

    class Meta:
        model = CustomUser
        fields = ["email", "password", "is_active", "is_admin"]


class CustomUserAdmin(BaseUserAdmin):
    form = UserChangeForm
    add_form = UserCreationForm

    list_display = [
        "id",
        "email",
        "is_admin",
        "is_verified",
        "is_trial",
        "is_subscription_active",
        "created_at",
        "updated_at",
    ]
    list_filter = ["is_admin", "is_verified"]
    fieldsets = [
        (None, {"fields": ["name", "email", "password"]}),
        (
            "Permissions",
            {
                "fields": [
                    "is_admin",
                    "is_verified",
                    "is_trial",
                    "is_subscription_active",
                ]
            },
        ),
    ]
    add_fieldsets = [
        (
            None,
            {
                "classes": ["wide"],
                "fields": ["email", "password1", "password2"],
            },
        ),
    ]
    search_fields = ["email"]
    ordering = ["-created_at", ]
    filter_horizontal = []


admin.site.register(CustomUser, CustomUserAdmin)
admin.site.unregister(Group)


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="cedric",
    version="1.0.32",
    author="Abhishek Choudhury",
    author_email="choudhuryabhishek76@gmail.com",
    description="Cedric is a Python library designed to streamline the process of setting up a Django application. ",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/cedric",
    packages=find_packages(),  # Use find_packages() directly
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["django", "requests", "art", "inquirer"],
    entry_points={"console_scripts": ["cedric-setup=src.cli:main"]},
)
